/****************************************************************************
 *			Tuning and Analysis Utilities
 *			http://www.cs.uoregon.edu/research/paracomp/tau
 ****************************************************************************
 * Copyright (c) 1997-2006
 *    Department of Computer and Information Science, University of Oregon
 *    Advanced Computing Laboratory, Los Alamos National Laboratory
 *    Research Center Juelich, ZAM Germany	
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Wyatt Spear - initial API and implementation
 ****************************************************************************/
package org.eclipse.ptp.tau.cdt.launch.internal;

import org.eclipse.cdt.debug.core.ICDTLaunchConfigurationConstants;
import org.eclipse.cdt.launch.internal.LocalRunLaunchDelegate;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.ptp.tau.core.internal.TAULaunchManage;


/**
 * 
 */
public class TAUCDTLaunchConfigurationDelegate extends LocalRunLaunchDelegate {
	
	private static final String buildText="Instrumenting and Building";
	private static final String launchText="Executing Instrumented Project";
	private static final String collectText="Collecting Performance Data";
	
	public void launch(ILaunchConfiguration configuration, String mode, ILaunch launchIn, IProgressMonitor monitor) throws CoreException
	{
		final ILaunch launch = launchIn;
		final TAULaunchManage tauManager = new TAULaunchManage(configuration,ICDTLaunchConfigurationConstants.ATTR_PROGRAM_NAME,ICDTLaunchConfigurationConstants.ATTR_PROJECT_NAME);
		
		Job tauBuild=new Job(buildText){

			protected IStatus run(IProgressMonitor monitor) {
				//System.out.println("Building");
				try {
					tauManager.TAUBuild(monitor);
				} catch (Exception e) {
					//e.printStackTrace();
					//System.out.println("Error Building");
					return new Status(IStatus.ERROR,"com.ibm.jdg2e.concurrency",IStatus.ERROR,"Build Incomplete",e);
				}
				//System.out.println("Done Building");
				return new Status(IStatus.OK,"com.ibm.jdg2e.concurrency",IStatus.OK,"Build Successful",null);
			}
		};
		
		final Job tauRun=new Job(launchText)
		{
			protected IStatus run(IProgressMonitor monitor) {
				//System.out.println("Executing");
				boolean runbuilt=tauManager.doRun();
				if(!runbuilt)
					return new Status(IStatus.WARNING,"com.ibm.jdg2e.concurrency",IStatus.WARNING,"Nothing to run",null);
				//super.launch(configuration, ILaunchManager.RUN_MODE, launch, monitor);
				LocalRunLaunchDelegate paraDel = new LocalRunLaunchDelegate();
				//mode = ILaunchManager.RUN_MODE;
				try {
					paraDel.launch(tauManager.getLaunchConfiguration(), ILaunchManager.RUN_MODE, launch, monitor);
					if(!launch.isTerminated())
					{
						if(!launch.canTerminate())
						{
							System.out.println("Launch can not terminate!  Possible infinate loop!");
							throw new OperationCanceledException();
						}
						while(!launch.isTerminated())
						{
							if(monitor.isCanceled())
							{
								tauManager.cleanup();
								throw new OperationCanceledException();
							}
							Thread.sleep(1000);
						}
					}
				} catch (Exception e) {
					//System.out.println("Error Executing");
					//e.printStackTrace();
					return new Status(IStatus.ERROR,"com.ibm.jdg2e.concurrency",IStatus.ERROR,"Execution Error",e);
					
				}
				//System.out.println("Done Executing");
				return new Status(IStatus.OK,"com.ibm.jdg2e.concurrency",IStatus.OK,"Execution Complete",null);
			}
		};
		
		final Job tauCollect=new Job(collectText){

			protected IStatus run(IProgressMonitor monitor) {
				//System.out.println("Collecting");
				try{
					tauManager.postlaunch(monitor);
				}catch(Exception e){
					//System.out.println("Error Collecting");
					return new Status(IStatus.ERROR,"com.ibm.jdg2e.concurrency",IStatus.ERROR,"Data Collection Error",e);
				}		
				//System.out.println("Done Collecting");
				return new Status(IStatus.OK,"com.ibm.jdg2e.concurrency",IStatus.OK,"Data Collected",null);
			}
		};

		JobChangeAdapter tauChange = new JobChangeAdapter(){
			public void done (IJobChangeEvent event){
				if(event.getJob().getName().equals(buildText)&&event.getResult().isOK())
				{
					tauRun.schedule();
				}
				else
				if(event.getJob().getName().equals(launchText)&&event.getResult().isOK())
				{
					tauCollect.schedule();
				}
			}
		};
		
		tauBuild.addJobChangeListener(tauChange);
		tauRun.addJobChangeListener(tauChange);
		tauBuild.schedule();
		
//		boolean runbuilt = false;
//		try {
//			tauManager.TAUBuild(monitor);
//		} catch (Exception e) {
//			//e.printStackTrace();
//			return;
//		}
//		runbuilt=tauManager.doRun();
//		if(!runbuilt)
//			return;
//		
//		//Launch the instrumented executable
//		super.launch(configuration, mode, launch, monitor);
//		//Collect data and clean up
//		try{
//		tauManager.postlaunch(monitor);
//		}catch(Exception e){
//			//e.printStackTrace();
//		}
	}		
}
