package org.eclipse.ptp.tau.cdt.launch.internal.ui;

import org.eclipse.cdt.core.model.ICProject;
import org.eclipse.cdt.debug.core.ICDTLaunchConfigurationConstants;
import org.eclipse.cdt.launch.internal.ui.LaunchMessages;
import org.eclipse.cdt.launch.internal.ui.LaunchUIPlugin;
import org.eclipse.cdt.launch.ui.CMainTab;
import org.eclipse.cdt.managedbuilder.core.IConfiguration;
import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ptp.tau.core.internal.ITAULaunchConfigurationConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class TAUCMainTab extends CMainTab implements ILaunchConfigurationTab{
	
	protected Combo projectCombo =null;
	protected Combo buildConfCombo=null;
	protected String projString=null;
	
	public TAUCMainTab() {
		super();
	}
	
	public TAUCMainTab(boolean x){
		super(x);
	}
	
	
	protected void createProjectGroup(Composite parent, int colSpan) {
		Composite projComp = new Composite(parent, SWT.NONE);
		GridLayout projLayout = new GridLayout();
		projLayout.numColumns = 2;
		projLayout.marginHeight = 0;
		projLayout.marginWidth = 0;
		projComp.setLayout(projLayout);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = colSpan;
		projComp.setLayoutData(gd);

		fProjLabel = new Label(projComp, SWT.NONE);
		fProjLabel.setText(LaunchMessages.getString("CMainTab.&ProjectColon")); //$NON-NLS-1$
		gd = new GridData();
		gd.horizontalSpan = 2;
		fProjLabel.setLayoutData(gd);

		fProjText = new Text(projComp, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		fProjText.setLayoutData(gd);
		fProjText.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent evt) {
				int bDex=buildConfCombo.getSelectionIndex();
				String bString=buildConfCombo.getText();
				initConfCombo();
				if(bDex>=0&&buildConfCombo.getItemCount()>bDex&&buildConfCombo.getItem(bDex).equals(bString))
					buildConfCombo.select(bDex);
				else
				if(buildConfCombo.getItemCount()>0)
					buildConfCombo.select(0);
				updateLaunchConfigurationDialog();
			}
		});

		fProjButton = createPushButton(projComp, LaunchMessages.getString("Launch.common.Browse_1"), null); //$NON-NLS-1$
		fProjButton.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent evt) {
				handleProjectButtonSelected();
				int bDex=buildConfCombo.getSelectionIndex();
				String bString=buildConfCombo.getText();
				initConfCombo();
				if(bDex>=0&&buildConfCombo.getItemCount()>bDex&&buildConfCombo.getItem(bDex).equals(bString))
					buildConfCombo.select(bDex);
				else
				if(buildConfCombo.getItemCount()>0)
					buildConfCombo.select(0);
				updateLaunchConfigurationDialog();
			}
		});
	}
	
	
	protected void createExeFileGroup(Composite parent, int colSpan) {
		Composite mainComp = new Composite(parent, SWT.NONE);
		GridLayout mainLayout = new GridLayout();
		mainLayout.numColumns = 1;
		mainLayout.marginHeight = 0;
		mainLayout.marginWidth = 0;
		mainComp.setLayout(mainLayout);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = colSpan;
		mainComp.setLayoutData(gd);
		fProgLabel = new Label(mainComp, SWT.NONE);
		fProgLabel.setText("C/C++/Fortran Build Configuration"); //LaunchMessages.getString("CMainTab.C/C++_Application")
		gd = new GridData();
		gd.horizontalSpan = 3;
		fProgLabel.setLayoutData(gd);
		buildConfCombo=new Combo(mainComp, SWT.DROP_DOWN | SWT.READ_ONLY
				| SWT.BORDER);
		//fProgText = new Text(mainComp, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		buildConfCombo.setLayoutData(gd);
		buildConfCombo.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent evt) {
				updateLaunchConfigurationDialog();
			}
		});
	}
	
	protected void initConfCombo()
	{
		//int seldex=buildConfCombo.getSelectionIndex();
		buildConfCombo.removeAll();
		ICProject project = getCProject();
		if (project == null) {
			MessageDialog.openInformation(getShell(), LaunchMessages.getString("CMainTab.Project_required"), //$NON-NLS-1$
					LaunchMessages.getString("CMainTab.Enter_project_before_searching_for_program")); //$NON-NLS-1$
			//buildConfCombo.removeAll();
			return;
		}
		
		IManagedBuildInfo info = ManagedBuildManager.getBuildInfo(project.getResource());
		if(info==null)
		{
			MessageDialog.openInformation(getShell(), LaunchMessages.getString("CMainTab.Project_required"), //$NON-NLS-1$
					LaunchMessages.getString("CMainTab.Enter_project_before_searching_for_program")); //$NON-NLS-1$
			//buildConfCombo.removeAll();
			return;
		}
		
		IConfiguration[] confs =info.getManagedProject().getConfigurations();
		
		for(int i=0;i<confs.length;i++)
		{
			buildConfCombo.add(confs[i].getName());
		}
		
	}
	
	public boolean isValid(ILaunchConfiguration config) {

		setErrorMessage(null);
		setMessage(null);

		String name = fProjText.getText().trim();
		if (name.length() == 0) {
			setErrorMessage(LaunchMessages.getString("CMainTab.Project_not_specified")); //$NON-NLS-1$
			return false;
		}
		if (!ResourcesPlugin.getWorkspace().getRoot().getProject(name).exists()) {
			setErrorMessage(LaunchMessages.getString("Launch.common.Project_does_not_exist")); //$NON-NLS-1$
			return false;
		}
		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(name);
		if (!project.isOpen()) {
			setErrorMessage(LaunchMessages.getString("CMainTab.Project_must_be_opened")); //$NON-NLS-1$
			return false;
		}

		name = buildConfCombo.getText();//fProgText.getText().trim();
		if (name==null||name.length() == 0) {
			setErrorMessage("Build configuration not specified");//LaunchMessages.getString("CMainTab.Program_not_specified") //$NON-NLS-1$
			return false;
		}
		
		String bcdne = "Build configuration does not exist";
		
		if (name.equals(".") || name.equals("..")) { //$NON-NLS-1$ //$NON-NLS-2$
			setErrorMessage(bcdne);//LaunchMessages.getString("CMainTab.Program_does_not_exist") //$NON-NLS-1$
			return false;
		}
		
		ICProject thisProject = getCProject();
		if (thisProject == null) {
			setErrorMessage(LaunchMessages.getString("Launch.common.Project_does_not_exist")); //$NON-NLS-1$
			return false;
		}
		
		IManagedBuildInfo info = ManagedBuildManager.getBuildInfo(thisProject.getResource());
		if(info==null)
		{
			setErrorMessage("Project has no valid build information"); //$NON-NLS-1$
			return false;
		}
		
		IConfiguration[] configs = info.getManagedProject().getConfigurations();
		
		if(configs.length<1)
		{
			setErrorMessage("No valid configurations"); //$NON-NLS-1$
			return false;
		}
		
		for(int i =0;i<configs.length;i++){
			if(configs[i].getName().equals(buildConfCombo.getText()))
				return true;
		}
		setErrorMessage(bcdne);//LaunchMessages.getString("CMainTab.Program_does_not_exist") //$NON-NLS-1$
		return false;
	}
	
	protected void updateProjectFromConfig(ILaunchConfiguration config) {
		String curProj=fProjText.getText();
		String projectName = EMPTY_STRING;
		try {
			projectName = config.getAttribute(ICDTLaunchConfigurationConstants.ATTR_PROJECT_NAME, EMPTY_STRING);
		} catch (CoreException ce) {
			LaunchUIPlugin.log(ce);
		}
		fProjText.setText(projectName);
		if(!curProj.equals(projectName))
			initConfCombo();
	}

	
	protected void updateProgramFromConfig(ILaunchConfiguration config) {
			String programName = EMPTY_STRING;
			try {
				programName = config.getAttribute(ITAULaunchConfigurationConstants.ATTR_TAUBUILD_CONFIGURATION_NAME, EMPTY_STRING);
			} catch (CoreException ce) {
				LaunchUIPlugin.log(ce);
			}
			if(!programName.equals(EMPTY_STRING))
				buildConfCombo.select(buildConfCombo.indexOf(programName));
			else
				buildConfCombo.select(0);
	}

	public void performApply(ILaunchConfigurationWorkingCopy config) {
		config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_PROJECT_NAME, fProjText.getText());
		config.setAttribute(ITAULaunchConfigurationConstants.ATTR_TAUBUILD_CONFIGURATION_NAME, buildConfCombo.getText());
		//System.out.println("Applying: "+buildConfCombo.getText());
		if (fTerminalButton != null) {
			config.setAttribute(ICDTLaunchConfigurationConstants.ATTR_USE_TERMINAL, fTerminalButton.getSelection());
		}
	}
	
}
